@@ -24,7 +24,9 @@ async fn main() -> Result<()> {
 
     let level = "info"; // if RUST_LOG not present, use `info` level
     tracing_subscriber::fmt()
-        .with_env_filter(EnvFilter::try_from_default_env().unwrap_or(EnvFilter::from(level)))
+        .with_env_filter(
+            EnvFilter::try_from_default_env().unwrap_or_else(|_| EnvFilter::from(level)),
+        )
         .init();
 
     run(&args, shutdown_rx).await
