@@ -12,13 +12,22 @@ pub trait Transport: Debug + Send + Sync {
     type Acceptor: Send + Sync;
     type Stream: 'static + AsyncRead + AsyncWrite + Unpin + Send + Sync + Debug;
 
-    async fn new(config: &TransportConfig) -> Result<Box<Self>>;
+    async fn new(config: &TransportConfig) -> Result<Self>
+    where
+        Self: Sized;
     async fn bind<T: ToSocketAddrs + Send + Sync>(&self, addr: T) -> Result<Self::Acceptor>;
     async fn accept(&self, a: &Self::Acceptor) -> Result<(Self::Stream, SocketAddr)>;
     async fn connect(&self, addr: &str) -> Result<Self::Stream>;
 }
 
 mod tcp;
-mod tls;
 pub use tcp::TcpTransport;
+#[cfg(feature = "tls")]
+mod tls;
+#[cfg(feature = "tls")]
 pub use tls::TlsTransport;
+
+#[cfg(feature = "noise")]
+mod noise;
+#[cfg(feature = "noise")]
+pub use noise::NoiseTransport;
